/**************************************************************************//**
 * @item     CosyOS Config
 * @file     mcucfg_8051.h
 * @brief    8051 Core Config File
 * @author   迟凯峰
 * @version  V3.3.3
 * @date     2023.07.24
 ******************************************************************************/

#ifndef __MCUCFG_8051_H
#define __MCUCFG_8051_H

/******************************************************************************
 *                             USER Definitions                               *
 ******************************************************************************/

          //*** <<< Use Configuration Wizard in Context Menu >>> ***//

///////////////////////////////////////////////////////////////////////////////
// <s> 标准头文件
// <i> 定义与您的MCU相匹配的标准头文件，您的c文件中可不必再包含此文件。
// <i> 示例一：STC8.H
// <i> 示例二：..\MCU\STC8.H，此种包含路径的写法，在图形用户界面输入时有时会出问题，建议在文本编辑界面中定义或加强校对。
#define MCUCFG_STANDARDHEAD             "STC8.H"
///////////////////////////////////////////////////////////////////////////////
// <o> 系统时钟
// <i> 告知CosyOS您所配置的系统时钟，单位为MHZ。
// <i> 由于定时器0采用12T模式，系统时钟最好是12的整数倍，以提高系统滴答周期的精度。
#define MCUCFG_SYSCLK                   24
#if !MCUCFG_SYSCLK
#error 非法的设置值！
#endif
///////////////////////////////////////////////////////////////////////////////
// <o> 最低优先级中断寄存器库
// <0=> bank0 <1=> bank1 <2=> bank2 <3=> bank3
// <i> 所有最低优先级中断的寄存器库，bank0为不使用独立的寄存器库。
#define MCUCFG_STKREGBANK               0
///////////////////////////////////////////////////////////////////////////////
// <q> 第二组DPTR寄存器
// <i> 告知CosyOS您是否启用第二组DPTR寄存器，如果您的MCU没有第二组DPTR寄存器，请勿勾选。
#define MCUCFG_DOUBLEDPTR               0
///////////////////////////////////////////////////////////////////////////////
// <h> PendSV_Handler
// <i> 由于51单片机没有PendSV_Handler软中断，您应专门配置一个硬件中断用于代替PendSV_Handler。
// <i> 您可选择一个未使用的硬件中断，并在初始化钩子中配置它，注意优先级应为最低级。

// <o> 中断号
// <i> 中断号
#define MCUCFG_PSVIRQ                   

// <o> 中断关闭
// <i> 此项您应在文本编辑界面中定义。
// <i> 示例：EX0 = 0
#define mPSV_Disable                    

// <o> 中断开启
// <i> 此项您应在文本编辑界面中定义。
// <i> 示例：EX0 = 1
#define mPSV_Enable                     

// <o> 中断触发
// <i> 置中断标志位
// <i> 此项您应在文本编辑界面中定义。
// <i> 示例：IE0 = 1
#define mPSV_Trigger                    

// <o> 中断清零
// <i> 清中断标志位
// <i> 此项您应在文本编辑界面中定义。
// <i> 示例：IE0 = 0
#define mPSV_Clear                      

// <o> 异步服务总数
// <i> 取值范围：<0~64>
// <i> 对于51单片机，您在中断中每调用一次异步服务，都要输入一个唯一的服务ID，此项用于定义您在中断中调用异步服务的总次数。
// <i> 示例：如果异步服务总数为10，那么您在中断中最多只能调用10次异步服务，服务ID为0~9。
#define MCUCFG_ISVTOTAL                 0

// </h>
///////////////////////////////////////////////////////////////////////////////
// <h> 动态内存设置
// <i> CosyOS会使用下列参数自动初始化内存池。

// <o> 内存池指针
// <i> 内存池的起始内存地址
#define MCUCFG_MALLOCMEMBPTR            4096
#if !MCUCFG_MALLOCMEMBPTR
#error 非法的设置值！
#endif

// <o> 内存池大小
// <i> 内存池的大小（字节数）
#define MCUCFG_MALLOCMEMSIZE            4096
#if !MCUCFG_MALLOCMEMSIZE
#error 非法的设置值！
#endif

// </h>
///////////////////////////////////////////////////////////////////////////////

                //*** <<< end of configuration section >>> ***//

/******************************************************************************
 *                               OS Definitions                               *
 ******************************************************************************/

/* Header */
#include <string.h>
#include <intrins.h>
#include MCUCFG_STANDARDHEAD
#include "..\System\os_typedef.h"

/* Memory */
#define _SYS_MEM_     data
#define _CODE_MEM_    code
#define _CONST_MEM_   code
#define _STACK_MEM_   idata
#ifndef _MALLOC_MEM_
#define _MALLOC_MEM_  xdata
#endif
#define _XDATA_MEM_   xdata
#define _DEBUG_MEM_   xdata

/* Register */
#define _SYS_REG_     _SYS_MEM_

/* Typedef */
typedef bit  tBIT;
typedef u8   tSP;
typedef u8   tStackSize;
typedef u8   tDM;
typedef u16  tPC;
typedef u32  tGRP;
typedef u16  tSysTick;
typedef u8   tFetion;

/* Extern */
extern bit vISV_FF;
extern tSP _SYS_MEM_ vMSP;
extern tSP _SYS_MEM_ vBSP;
extern u16 vISS[MCUCFG_ISVTOTAL];
extern void __isv_handler(void);

/* * */
#define MCUCFG_MCUARC           !__ARM__
#define MCUCFG_PCLEN            2
#define MCUCFG_TASKSTACKMONITOR __DISABLED__
#if MCUCFG_STKREGBANK
#define __USING__               using MCUCFG_STKREGBANK
#define __REGSIZE__             8
#define __BASICSTACKSIZE__      (MCUCFG_DOUBLEDPTR ? 9 : 7)
#else
#define __USING__
#define __REGSIZE__             0
#define __BASICSTACKSIZE__      (MCUCFG_DOUBLEDPTR ? 17 : 15)
#endif
#define __C51USING__            __USING__
#define __STK_ATTRIBUTE__       interrupt 1 __USING__
#define __PSV_ATTRIBUTE__       interrupt MCUCFG_PSVIRQ __USING__
#define __251_REENTRANT__
#define __BSP_ALIGN__
#define MCUCFG_DIRMSGTYPE       0
#define __DM_PSP__
#define __DM_SIZE__             (u8)(&mx - &m0 - 1)
#if SYSCFG_DIRMSGMODE == __PERFORMANCE__
#define __DM_VAR__              tDM m0_
#define __DM_VAL__              1
#elif SYSCFG_DIRMSGMODE == __INTELLIGENT__
#define __DM_VAR__              tDM R3_, tDM R5_, tDM R7_, tDM m0_
#define __DM_VAL__              0, 0, 0, 1
#endif
#define __STACKSIZE_TASKMGR__   (SYSCFG_TASKCREATEMODE ? __BASICSTACKSIZE__ : __BASICSTACKSIZE__ + 16)
#define __STACKSIZE_DEBUGGER__  (SYSCFG_TASKCREATEMODE ? __BASICSTACKSIZE__ : __BASICSTACKSIZE__ + 16)
#define __STACKSIZE_STARTER__   (SYSCFG_TASKCREATEMODE ? __BASICSTACKSIZE__ : __BASICSTACKSIZE__ + 16)
#define __STACKSIZE_SYSIDLE__   (SYSCFG_TASKCREATEMODE ? __BASICSTACKSIZE__ : __BASICSTACKSIZE__ + 16)

/* API */
#define mSysTick_Cycle        ((1UL * MCUCFG_SYSCLK * SYSCFG_STKCYCLE) / 12)
#if !mSysTick_Cycle || mSysTick_Cycle > 65536
#error 系统滴答周期溢出！
#endif
#define mSysTick_InitValue    (65536 - mSysTick_Cycle)
#define mSysTick_Counter      ((TH0 << 8) | TL0)
#define mSTK_Disable          ET0 = 0
#define mSTK_Enable           ET0 = 1
#define mTaskNode_Head_       tStackSize stacklen;
#if __REGSIZE__
#define mTaskNode_Tail_       u8 reg[__REGSIZE__];
#define mReg_PUSH	\
	*(u32 *)(vTASKING->reg + 0) = *(u32 _SYS_MEM_ *)0;	\
	*(u32 *)(vTASKING->reg + 4) = *(u32 _SYS_MEM_ *)4
#define mReg_POP	\
	*(u32 _SYS_MEM_ *)0 = *(u32 *)(vTASKING->reg + 0);	\
	*(u32 _SYS_MEM_ *)4 = *(u32 *)(vTASKING->reg + 4)
#else
#define mTaskNode_Tail_
#define mReg_PUSH do{}while(false)
#define mReg_POP  do{}while(false)
#endif

#define mDisableIRQ	EA = 0
#define mEnableIRQ	EA = 1

#define mEnterCritical	\
do{	\
	mPSV_Disable;	\
	mSTK_Disable;	\
}while(false)

#define mExitCritical	\
do{	\
	mSTK_Enable;	\
	mPSV_Enable;	\
}while(false)

#define mSys_Idle	\
do{	\
	PCON |= 0x01;	\
	OS_NOPx4;	\
}while(false)

#define mSys_INIT	\
do{	\
	vMSP = SP;	\
	vBSP = SP + 1;	\
	__init_mempool((void _MALLOC_MEM_ *)MCUCFG_MALLOCMEMBPTR, MCUCFG_MALLOCMEMSIZE);	\
	OS_NOPx1;	\
	AUXR &=~0x80;   \
	TMOD &= 0xF0;	\
	TL0 = (u8)(mSysTick_InitValue);	\
	TH0 = (u8)(mSysTick_InitValue >> 8);	\
	TR0 = 1;	\
	mSTK_Enable;	\
	mPSV_Enable;	\
	EA = 1;	\
}while(false)

#define mTaskmgr_Counting	\
	counter2 = (counter1 * 100 * 12) / MCUCFG_SYSCLK / counter2

#define mSTK_Counting	\
do{	\
	if(mSysTick_Counter <= tick_temp) break;	\
	vSTK_Counter1 += mSysTick_Counter - tick_temp;	\
	vSTK_Counter2++;	\
}while(false)

#define mScheduler_INIT	\
	tStackSize _SYS_REG_ stacklen;	\
	u8 _STACK_MEM_  * _SYS_REG_ msp8;	\
	u8 _MALLOC_MEM_ * _SYS_REG_ psp8;	\
	vScheduling_f = false

#define mTaskStack_INIT	\
do{	\
	*(u16 *)node_news->BSP = ((u16)vACTBUF->entry << 8) | ((u16)vACTBUF->entry >> 8);	\
	*(u8 *)(node_news->BSP + (MCUCFG_DOUBLEDPTR ? 9 : 7) - 1) = 0;	\
	node_news->stacklen = __BASICSTACKSIZE__;	\
}while(false)

#define mTaskStack_LEN	\
	stacklen = SP - vMSP

#define mEvery_Monitor do{}while(false)

#if SYSCFG_SAFERUNTIME == __ENABLED__
#define mSRT_Monitor    \
if(vTASKING->TPL > node_news->TPL)	\
{	\
	vTASKING->counter = 0;	\
}
#else
#define mSRT_Monitor
#endif

#if SYSCFG_DEBUGGING == __ENABLED__
#if SYSCFG_TASKCREATEMODE == __STATIC__
#define mPUSH_Monitor	\
do{	\
	if(vTASKING->stacksize < stacklen)	\
	{	\
		vTASKING->state = __STOPPED_TSOF__;    \
		vFault.overflow_taskstack = true;  \
	}	\
	else	\
	{	\
		push_f = true;	\
		mSRT_Monitor    \
	}	\
}while(false)
#else
#define mPUSH_Monitor	\
do{	\
	if(vTASKING->stacksize < stacklen)	\
	{	\
		vTASKING->stacksize = stacklen;	\
		__free(vTASKING->BSP);	\
		vTASKING->BSP = NULL;	\
		vTASKING->BSP = (u8 *)__malloc(vTASKING->stacksize);	\
		if(vTASKING->BSP != NULL)	\
		{	\
			vTASKING->realloc = true;	\
			push_f = true;	\
		}	\
		else	\
		{	\
			vTASKING->state = __STOPPED_TSRF__;	\
			vFault.reallocfail_taskstack = true;	\
		}	\
	}	\
	else	\
	{	\
		push_f = true;	\
	}	\
	if(push_f)	\
	{	\
		mSRT_Monitor    \
	}	\
}while(false)
#endif
#else
#if SYSCFG_TASKCREATEMODE == __STATIC__
#define mPUSH_Monitor	\
do{	\
	if(vTASKING->stacksize < stacklen)	\
	{	\
		vTASKING->state = __STOPPED_TSOF__;    \
	}	\
	else	\
	{	\
		push_f = true;	\
		mSRT_Monitor    \
	}	\
}while(false)
#else
#define mPUSH_Monitor	\
do{	\
	if(vTASKING->stacksize < stacklen)	\
	{	\
		vTASKING->stacksize = stacklen;	\
		__free(vTASKING->BSP);	\
		vTASKING->BSP = NULL;	\
		vTASKING->BSP = (u8 *)__malloc(vTASKING->stacksize);	\
		if(vTASKING->BSP != NULL)	\
		{	\
			push_f = true;	\
		}	\
		else	\
		{	\
			vTASKING->state = __STOPPED_TSRF__;	\
		}	\
	}	\
	else	\
	{	\
		push_f = true;	\
	}	\
	if(push_f)	\
	{	\
		mSRT_Monitor    \
	}	\
}while(false)
#endif
#endif

#if SYSCFG_TASKPCMONITOR == __ENABLED__
#define mTaskPC_Monitor	\
do{	\
	if(!vTaskmgrBinary) break;	\
	vPC = *(tPC *)(SP + 1 - __BASICSTACKSIZE__);	\
	vPC = (vPC << 8) | (vPC >> 8);	\
}while(false)
#else
#define mTaskPC_Monitor do{}while(false)
#endif

#define mUsedTime_END	\
do{	\
	if(usedtime[0])	\
	{	\
		usedtime[0]--;	\
		usedtime[1] = 65536 - usedtime[1] + counter - mSysTick_InitValue;	\
	}	\
	else	\
	{	\
		if(counter >= usedtime[1])	\
		{	\
			usedtime[1] = counter - usedtime[1];	\
		}	\
		else	\
		{	\
			usedtime[1] = 65536 - usedtime[1] + counter - mSysTick_InitValue;	\
		}	\
	}	\
	vTASKING->usedtime[0] += usedtime[0];	\
	vTASKING->usedtime[0] += (vTASKING->usedtime[1] + usedtime[1]) / mSysTick_Cycle;	\
	vTASKING->usedtime[1]  = (vTASKING->usedtime[1] + usedtime[1]) % mSysTick_Cycle;	\
}while(false)

#define mUsedTime_INIT	\
do{	\
	usedtime[0] = 0;	\
	usedtime[1] = counter;	\
}while(false)

#define mTaskStack_PUSH	\
do{	\
	msp8 = (u8 _STACK_MEM_  *)vBSP;	\
	psp8 = (u8 _MALLOC_MEM_ *)vTASKING->BSP;	\
	vTASKING->stacklen = stacklen;	\
	do{	\
		*psp8++ = *msp8++;	\
	}while(--stacklen);	\
	mReg_PUSH;	\
	mTaskPC_Monitor;	\
}while(false)

#define mTaskStack_POP	\
do{	\
	vTASKING = node_news;	\
	msp8 = (u8 _STACK_MEM_  *)vBSP;	\
	psp8 = (u8 _MALLOC_MEM_ *)vTASKING->BSP;	\
	stacklen = vTASKING->stacklen;	\
	SP = vMSP + stacklen;	\
	do{	\
		*msp8++ = *psp8++;	\
	}while(--stacklen);	\
	mReg_POP;	\
}while(false)

#define mISV_Do(svid)	\
do{	\
	extern bit vISV_F##svid;	\
	vISS[svid] = (u16)&isv_;	\
	vISV_F##svid = true;	\
	vISV_FF = true;	\
	mPSV_Trigger;	\
}while(false)

#define mISV	\
	mPSV_Clear;	\
	if(vISV_FF) __isv_handler()

#define miWriteFlagBits	\
	static bit flag = false;	\
	if(!flag)	\
	{	\
		flag = true

#endif
